# Stuart J Macdonald, 11 March 2021

# LOAD bunch of packages use below
library(ggplot2)
library(RColorBrewer)
library(gridExtra)
library(DescTools)


# DEFINE DIRECTORIES
# -> Upper-level
# -> Pair of RNAi directories (one for each reciprocal cross direction)
uberDir <- "/Users/sjmac/Dropbox/sjm_dropbox_docs/Caffeine_pA_Cage/"
rna1_dir <- paste(uberDir,"RNAi/FilteredData/SummaryData/",sep="")
rna2_dir <- paste(uberDir,"RNAi_Recip/FilteredData/SummaryData/",sep="")


# READ IN filtered, tube-by-tube (individual-by-individual) phenotype files
rna1_pheno <- read.table(file=paste(rna1_dir,"TruncTubeData.txt",sep=""),header=T,sep="\t")
rna2_pheno <- read.table(file=paste(rna2_dir,"TruncTubeData.txt",sep=""),header=T,sep="\t")

	# QUICKLY make sure the pair of files are formatted the same
	colnames(rna1_pheno)==colnames(rna2_pheno)
	# [1] TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE TRUE
	dim(rna1_pheno)
	# [1] 1272   14
	dim(rna2_pheno)
	# [1] 928  14


# MERGE (which can do since genotype names independent)
# REMOVE UNECESSARY columns (which is most of them)
pheno <- rbind(rna1_pheno,rna2_pheno)
pheno <- pheno[,-c(1,2,5,6,7,9,10,11,13,14)]

	# KEEPING these 4 columns
		head(pheno)
	#           Directory      Monitor                                          RIL Lifespan.hours
	# 1 damoutput20201205 Monitor1.txt GAL4_1099.KK_107641_CG3050_Cyp6d5.No_Unknown       33.03333
	# 2 damoutput20201205 Monitor1.txt GAL4_1099.KK_107641_CG3050_Cyp6d5.No_Unknown       52.33333
	# 3 damoutput20201205 Monitor1.txt GAL4_1099.KK_107641_CG3050_Cyp6d5.No_Unknown       24.75000
	# 4 damoutput20201205 Monitor1.txt GAL4_1099.KK_107641_CG3050_Cyp6d5.No_Unknown       82.66667
	# 5 damoutput20201205 Monitor1.txt        GAL4_1099.KK_100545_CG2788_Dot.Yes_No      102.78333
	# 6 damoutput20201205 Monitor1.txt        GAL4_1099.KK_100545_CG2788_Dot.Yes_No      131.55000


# CONVERT 'Directory' column to vector code (1 or 2
# -> Since each 'block' of the experiment (i.e. a reciprocal cross)
#    was done on 1 day, the directory information is equivalent to block
# -> So this allows the 2 blocks to be more easily marked
pheno[,"Directory"][pheno[,"Directory"]=="damoutput20201205"] <- 1
pheno[,"Directory"][pheno[,"Directory"]=="damoutput20210113"] <- 2
colnames(pheno)[1] <- "Block"


###########
# PLAN is to make 4 boxplots, one per GAL4 (gut/1099 and ubiq/25374)
# and cross direction (block 1/2) combination
###########


###########
# STEP 1
# Keep only the genotypes I care about (block 1 particularly, but
# also block 2 to a lesser extent, tested lots more genotypes than
# are relevant for the X-QTL paper)


# DEFINE object to put reformatted data in
filtered_pheno <- NULL


# PURGE rows to only keep desired genotypes
# -> Did matrix(unique(pheno$RIL),ncol=1) and then made
#    this list manually
# -> Only keeping X-QTL paper-relevant genotypes
# -> And sorting the genotypes by the L-to-R (X-3R)
#    order of the QTL locations the genes reside in
To_Keep_Genos <- c("GAL4_1099.TK_60000_GD_Control.NA_NA",
	"GAL4_1099.GD_46424_CG34412_Tlk.NA_NA",
	"GAL4_1099.GD_2620_CG3327_E23.NA_NA",
	"GAL4_1099.GD_9489_CG2788_Dot.NA_NA",
	"GAL4_1099.GD_37736_CG16963_Crys.NA_NA",
	"GAL4_1099.GD_6121_CG12602_Vha100-5.NA_NA",
	"GAL4_1099.GD_50507_CG33503_Cyp12d1.NA_NA",
	"GAL4_1099.GD_12138_CG3050_Cyp6d5.NA_NA",
	"GAL4_1099.GD_38661_CG33970_osy.NA_NA",
	"GAL4_25374.TK_60000_GD_Control.NA_NA",
	"GAL4_25374.GD_2620_CG3327_E23.NA_NA",
	"GAL4_25374.GD_9489_CG2788_Dot.NA_NA",
	"GAL4_25374.GD_37736_CG16963_Crys.NA_NA",
	"GAL4_25374.GD_6121_CG12602_Vha100-5.NA_NA",
	"GAL4_25374.GD_50507_CG33503_Cyp12d1.NA_NA",
	"GAL4_25374.GD_12138_CG3050_Cyp6d5.NA_NA",
	"TK_60000_GD_Control.NA_NA.GAL4_1099",
	"GD_46424_CG34412_Tlk.NA_NA.GAL4_1099",
	"GD_2620_CG3327_E23.NA_NA.GAL4_1099",
	"GD_9489_CG2788_Dot.NA_NA.GAL4_1099",
	"GD_37736_CG16963_Crys.NA_NA.GAL4_1099",
	"GD_6121_CG12602_Vha100-5.NA_NA.GAL4_1099",             
	"GD_50507_CG33503_Cyp12d1.NA_NA.GAL4_1099",
	"GD_12138_CG3050_Cyp6d5.NA_NA.GAL4_1099",
	"GD_38661_CG33970_osy.NA_NA.GAL4_1099",
	"TK_60000_GD_Control.NA_NA.GAL4_25374",
	"GD_2620_CG3327_E23.NA_NA.GAL4_25374",
	"GD_9489_CG2788_Dot.NA_NA.GAL4_25374",
	"GD_37736_CG16963_Crys.NA_NA.GAL4_25374",
	"GD_6121_CG12602_Vha100-5.NA_NA.GAL4_25374",     
	"GD_50507_CG33503_Cyp12d1.NA_NA.GAL4_25374",
	"GD_12138_CG3050_Cyp6d5.NA_NA.GAL4_25374")


# DEFINE a 1:4 code allowing subsequent splitting of the
# genotypes for driver- and cross-specific plots
CODE_driver_by_cross <- c(rep(1,9),rep(2,7),rep(3,9),rep(4,7))


# DEFINE a numeric code for each genotype such
# that ordering genotypes within each of the to-be-
# generated plots is easy
CODE_geno_order <- c(1:9,1:7,1:9,1:7)


# DEFINE a numeric code to distinguish the control
# genotype (i.e. TK_60000) from the UAS-RNAi genotypes
CODE_control_or_UAS <- c(1,rep(2,8),1,rep(2,6),1,rep(2,8),1,rep(2,6))


# LOOP through each genotype, extract data,
# define some codes (for subsequent plots), and re-merge
for(i in 1:length(To_Keep_Genos)) {

	# SET the current genotype
	# EXRACT data associated with it
	cur_geno <- To_Keep_Genos[i]
	cur_data_tmp <- pheno[match(pheno[,"RIL"],cur_geno,nomatch=0)!=0,]
	
	# CONVERT information in 'Monitor' column to a
	# numeric vector (1:N, where N is the maximum number
	# of monitors the genotype was tested over)
	monitor_rep <- as.numeric(as.factor(cur_data_tmp[,"Monitor"]))
	
	# GENERATE a code defining which plot the genotype
	# should be presented in
	which_plot <- rep(CODE_driver_by_cross[i],length(monitor_rep))

	# GENERATE a code defining whether the genotype is
	# a control or a UAS-RNAi
	which_control <- rep(CODE_control_or_UAS[i],length(monitor_rep))

	# GENERATE a code numerically-defining genotype
	# allowing a specific genotype order in plots
	genotype_order <- rep(CODE_geno_order[i],length(monitor_rep))
	
	# MERGE
	# -> From 'cur_data_tmp' take just the genotype ID and the phenotype
	# -> Then combine with the 4 numeric vectors just made
	cur_data_out <- cbind(cur_data_tmp[,-c(1,2)],
		monitor_rep,which_plot,which_control,genotype_order)
	
	# APPEND to current output data
	filtered_pheno <- rbind(filtered_pheno,cur_data_out)
}


# SIMPLIFY 'filtered_pheno' column headers and row numbers
rownames(filtered_pheno) <- 1:nrow(filtered_pheno)
colnames(filtered_pheno)[1:2] <- c("full_geno_name","pheno")


###########
# STEP 2
# Split the 'filtered_pheno' object into 4 based on the 'which_plot'
# column, then execute Dunnett's tests comparing all UAS-RNAi against
# each of the relevant controls


# SPLIT into 4 files
# -> at same time 'as.data.frame' them
p1_1099_mGAL4_by_fUAS <- as.data.frame(filtered_pheno[filtered_pheno[,"which_plot"]==1,])
p2_25374_mGAL4_by_fUAS <- as.data.frame(filtered_pheno[filtered_pheno[,"which_plot"]==2,])
p3_1099_fGAL4_by_mUAS <- as.data.frame(filtered_pheno[filtered_pheno[,"which_plot"]==3,])
p4_25374_fGAL4_by_mUAS <- as.data.frame(filtered_pheno[filtered_pheno[,"which_plot"]==4,])

	# BRIEF check that all data still here
	dim(p1_1099_mGAL4_by_fUAS)
	# [1] 285   6
	dim(p2_25374_mGAL4_by_fUAS)
	# [1] 224   6
	dim(p3_1099_fGAL4_by_mUAS)
	# [1] 441   6
	dim(p4_25374_fGAL4_by_mUAS)
	# [1] 345   6
	# 285+224+441+345
	# [1] 1295
	dim(filtered_pheno)
	# [1] 1295    6


# DUNNETT'S tests
# -> Leverages the 'genotype_order' column, where every CON is 1 and
#    all UAS-RNAi are >1, to do the repeated tests to control
p1_DT <- DunnettTest(x=p1_1099_mGAL4_by_fUAS$pheno,g=p1_1099_mGAL4_by_fUAS$genotype_order)
p2_DT <- DunnettTest(x=p2_25374_mGAL4_by_fUAS$pheno,g=p2_25374_mGAL4_by_fUAS$genotype_order)
p3_DT <- DunnettTest(x=p3_1099_fGAL4_by_mUAS$pheno,g=p3_1099_fGAL4_by_mUAS$genotype_order)
p4_DT <- DunnettTest(x=p4_25374_fGAL4_by_mUAS$pheno,g=p4_25374_fGAL4_by_mUAS$genotype_order)

	p1_DT
	#   Dunnett's test for comparing several treatments with a control :  
	#     95% family-wise confidence level
	# 
	# $`1`
	#           diff    lwr.ci     upr.ci    pval    
	# 2-1 -26.745934 -41.15875 -12.333119 1.0e-05 ***
	# 3-1 -33.869271 -48.16724 -19.571300 5.8e-09 ***
	# 4-1  -6.275504 -20.68832   8.137311 0.78992    
	# 5-1  -7.079167 -21.37714   7.218804 0.67558    
	# 6-1 -24.898085 -39.31090 -10.485269 4.5e-05 ***
	# 7-1 -24.867708 -39.16568 -10.569738 3.8e-05 ***
	# 8-1 -22.098958 -36.39693  -7.800988 0.00042 ***
	# 9-1 -11.279167 -25.57714   3.018804 0.18970    
	# 
	# ---
	# Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

	p2_DT
	#   Dunnett's test for comparing several treatments with a control :  
	#     95% family-wise confidence level
	# 
	# $`1`
	#           diff     lwr.ci     upr.ci   pval    
	# 2-1 -21.460417 -36.954956 -5.9658772 0.0023 ** 
	# 3-1 -12.977604 -28.472144  2.5169353 0.1369    
	# 4-1   2.916146 -12.578394 18.4106853 0.9928    
	# 5-1   5.751042  -9.743498 21.2455811 0.8450    
	# 6-1 -16.350000 -31.844539 -0.8554606 0.0341 *  
	# 7-1 -22.886458 -38.380998 -7.3919189 0.0011 ** 
	# 
	# ---
	# Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

	p3_DT
	#   Dunnett's test for comparing several treatments with a control :  
	#     95% family-wise confidence level
	# 
	# $`1`
	#           diff     lwr.ci      upr.ci    pval    
	# 2-1  -9.220723 -19.067422   0.6259757  0.0784 .  
	# 3-1 -16.995399 -26.659736  -7.3310623 2.4e-05 ***
	# 4-1  -3.350608 -13.014945   6.3137294  0.9348    
	# 5-1  -5.701997 -15.366334   3.9623405  0.5124    
	# 6-1 -12.180122 -21.844459  -2.5157845  0.0057 ** 
	# 7-1 -27.578895 -37.301804 -17.8559859 6.4e-13 ***
	# 8-1 -18.608775 -28.392429  -8.8251206 4.7e-06 ***
	# 9-1   5.483516  -4.239393  15.2064255  0.5646    
	# 
	# ---
	# Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

	p4_DT
	#   Dunnett's test for comparing several treatments with a control :  
	#     95% family-wise confidence level
	# 
	# $`1`
	#            diff    lwr.ci      upr.ci    pval    
	# 2-1 -26.2468807 -42.52345  -9.9703066 0.00020 ***
	# 3-1 -19.4993112 -35.77589  -3.2227372 0.01112 *  
	# 4-1  -0.4263946 -16.70297  15.8501795 1.00000    
	# 5-1 -26.1497511 -42.72718  -9.5723271 0.00031 ***
	# 6-1 -42.0055982 -58.37879 -25.6324076 4.1e-10 ***
	# 7-1 -17.0826446 -33.35922  -0.8060705 0.03525 *  
	# 
	# ---
	# Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1


# DEFINE color vectors based on Dunnett's tests statistical significance
# -> level 1 = control = "gray75"
# -> level 2 = ns (>0.05) = "white"
# -> level 3 = sig at <0.05 = "slategray1"
# -> level 4 = sig at <0.005 = "dodgerblue1"
# -> (since Bonferroni is 0.05/8=0.00625 or 0.05/6=0.0083 [depending
#    on the GAL4] this color scheme basically reflects moderately
#    significant versus significant after correction for multiple testing

color_levels_A <- c("gray75","white","slategray1","dodgerblue1") # desired colors for each level (p2,p3,p4)
color_levels_B <- c("gray75","white","dodgerblue1") # desired colors for each level (p1 - which doesn't have all levels)


p1_geno_color_codes <- c(1,4,4,2,2,4,4,4,2) # per genotype level
p2_geno_color_codes <- c(1,4,2,2,2,3,4)
p3_geno_color_codes <- c(1,2,4,2,2,3,4,4,2)
p4_geno_color_codes <- c(1,4,3,2,4,4,3)


p1_colors_indiv <- unlist(lapply(p1_1099_mGAL4_by_fUAS$genotype_order,
	function(XXX) p1_geno_color_codes[XXX])) # per individual level
p2_colors_indiv <- unlist(lapply(p2_25374_mGAL4_by_fUAS$genotype_order,
	function(XXX) p2_geno_color_codes[XXX])) # per individual level
p3_colors_indiv <- unlist(lapply(p3_1099_fGAL4_by_mUAS$genotype_order,
	function(XXX) p3_geno_color_codes[XXX])) # per individual level
p4_colors_indiv <- unlist(lapply(p4_25374_fGAL4_by_mUAS$genotype_order,
	function(XXX) p4_geno_color_codes[XXX])) # per individual level


###########
# STEP 3
# Draw plots

# PLOT 1
p1 <- ggplot(p1_1099_mGAL4_by_fUAS,aes(x=as.factor(genotype_order),y=pheno)) + 
	geom_boxplot(aes(fill=as.factor(p1_colors_indiv)),show.legend=FALSE,outlier.shape=NA) +
	scale_fill_manual(values = color_levels_B) +
	geom_jitter(shape=16,color=rgb(0,0,0,alpha=0.5),size=1,position=position_jitter(0.2)) +
	theme_classic(base_size=12) +
	theme(axis.text.x=element_text(angle=45,hjust=1),plot.title=element_text(size=12)) +
#	xlab("Gene Tested") +
	xlab(NULL) +
	ylab("Lifespan (hrs)") +
	ggtitle("A. Gut (Gal4*UAS)") +
	scale_x_discrete(breaks=1:9,
        labels=c("CON","Tlk","E23","Ugt36A1","Crys","Vha100-5","Cyp12d1","Cyp6d5","osy"))


# PLOT 2
p2 <- ggplot(p2_25374_mGAL4_by_fUAS,aes(x=as.factor(genotype_order),y=pheno)) + 
	geom_boxplot(aes(fill=as.factor(p2_colors_indiv)),show.legend=FALSE,outlier.shape=NA) +
	scale_fill_manual(values = color_levels_A) +
	geom_jitter(shape=16,color=rgb(0,0,0,alpha=0.5),size=1,position=position_jitter(0.2)) +
	theme_classic(base_size=12) +
	theme(axis.text.x=element_text(angle=45,hjust=1),plot.title=element_text(size=12)) +
#	xlab("Gene Tested") +
	xlab(NULL) +
#	ylab("Lifespan (hrs)") +
	ylab(NULL) +
	ggtitle("B. Ubiq (Gal4*UAS)") +
	scale_x_discrete(breaks=1:7,
        labels=c("CON","E23","Ugt36A1","Crys","Vha100-5","Cyp12d1","Cyp6d5"))


# PLOT 3
p3 <- ggplot(p3_1099_fGAL4_by_mUAS,aes(x=as.factor(genotype_order),y=pheno)) + 
	geom_boxplot(aes(fill=as.factor(p3_colors_indiv)),show.legend=FALSE,outlier.shape=NA) +
	scale_fill_manual(values = color_levels_A) +
	geom_jitter(shape=16,color=rgb(0,0,0,alpha=0.5),size=1,position=position_jitter(0.2)) +
	theme_classic(base_size=12) +
	theme(axis.text.x=element_text(angle=45,hjust=1),plot.title=element_text(size=12)) +
	xlab("Gene Tested") +
	ylab("Lifespan (hrs)") +
	ggtitle("C. Gut (UAS*Gal4)") +
	scale_x_discrete(breaks=1:9,
        labels=c("CON","Tlk","E23","Ugt36A1","Crys","Vha100-5","Cyp12d1","Cyp6d5","osy"))


# PLOT 4
p4 <-ggplot(p4_25374_fGAL4_by_mUAS,aes(x=as.factor(genotype_order),y=pheno)) + 
	geom_boxplot(aes(fill=as.factor(p4_colors_indiv)),show.legend=FALSE,outlier.shape=NA) +
	scale_fill_manual(values = color_levels_A) +
	geom_jitter(shape=16,color=rgb(0,0,0,alpha=0.5),size=1,position=position_jitter(0.2)) +
	theme_classic(base_size=12) +
	theme(axis.text.x=element_text(angle=45,hjust=1),plot.title=element_text(size=12)) +
	xlab("Gene Tested") +
#	ylab("Lifespan (hrs)") +
	ylab(NULL) +
	ggtitle("D. Ubiq (UAS*Gal4)") +
	scale_x_discrete(breaks=1:7,
        labels=c("CON","E23","Ugt36A1","Crys","Vha100-5","Cyp12d1","Cyp6d5"))


# DEFINE PLOT GRID
lay <- rbind(c(1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2), # a grid that, along with lack of y-lab
             c(3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4)) # in right plots, makes bars all same width

# OUTPUT PLOT
#quartz(width=6.3,heigh=5.3)
pdf(paste(uberDir,"RNAi_Fig_20210311.pdf",sep=""),width=6.3,heigh=5.3)
grid.arrange(p1,p2,p3,p4,layout_matrix=lay) # actually do the plot
graphics.off()





