# Stuart J Macdonald, 9 March 2021
#
# Series of VDRC UAS-RNAi strains along with a ubiquitous GAL4
# (Act5C, 25374) and an anterior midgut GAL4 (1099)
#
# 1% caffeine
#
# All aspirated by SJM
#
# Subset of genotypes used in "RNAi" _and_ used reciprocal crosses

############################
###       GLOBAL         ###
###      VARIABLES       ###
############################

# DIRECTORIES
# -> Upper-level
# -> RIL duplication files
# -> Monitor initiation files
# -> Monitor output directories
# -> Summary data OUTPUT
# -> Activity plot OUTPUT
uberDir <- "/Users/sjmac/Dropbox/sjm_dropbox_docs/Caffeine_pA_Cage/RNAi_Recip/FilteredData/"
dirRILdup <- paste(uberDir,"RILduplication/",sep="")
dirInit <- paste(uberDir,"DAMinitiation/",sep="")
dirOut <- paste(uberDir,"DAMoutput/",sep="")
dirSummary <- paste(uberDir,"SummaryData/",sep="")
dirActivityBarplot <- paste(uberDir,"ActivityPlots/",sep="")


# NAMING SCHEME
# -> Stem for initiation files
# -> Stem for monitor output directories
stemInitFiles <- "daminitiation"
stemDAMdir <- "damoutput"


# LOAD/VALIDATE MONITOR FILES
# -> Set monitor loading format
#    "2halves" = A/B side loading
#    "4quadrants" = 1/2/3/4 quandrant loading
# -> Set # columns, and whether to strip any
# -> Set monitor reading interval
# -> Set error file
#    ONLY purpose of this file is to allow a subsequent
#    manual check of any raw monitor files for
#    problems occuring during data collection
monitorFormat <- "2halves"
monitorFileNCols <- 42
stripCols <- "no"
readInterval <- 1
readIntUnits <- "mins"
ErrorFile <- paste(uberDir,"MonitorErrors.txt",sep="")
write(paste("Dir","Monitor","LessThanInterval","DoubleInterval",
    "MoreThanDouble",sep="\t"),ErrorFile,append=F)


# THRESHOLDS
# -> activityThresh - min number activity
#    counts for usable tube
# -> aliveThresh - max number live flies
#    for a genotype at end of run
# -> flyThresh - min number tested flies
#    per genotype
# DETERMINE WHEN FLY IS DEAD
# -> MaxNumSequentialInactiveBins - acceptable
#    number zero activity bins between active
#    bins (720min = 12hrs)
# -> MaxNumActiveBins - number active bins
#    allowed following an inactive period and
#    still consider fly dead at start of that
#    inactive period
# DETERMINE IF FLY ALIVE AT END OF RUN
# -> EndTestWindow - test period at end of run
#    to examine (120min = 2hrs)
# -> EndActivityCount - acceptable number of
#    counts to still consider fly dead
activityThresh <- 100
aliveThresh <- 0
flyThresh <- 7
MaxNumSequentialInactiveBins <- 720
#MaxNumSequentialInactiveBins <- 360
#MaxNumSequentialInactiveBins <- 480
#MaxNumSequentialInactiveBins <- 300
MaxNumActiveBins <- 5    # More than 7 for this parameter seems to have no important effect
#MaxNumActiveBins <- 7
#MaxNumActiveBins <- 10
EndTestWindow <- 120
EndActivityCount <- 5





############################
###       GENERIC        ###
###         CODE         ###
############################

# GATHER data on every monitor initiated in experiment
MonitorInfo <- CollateInitiationFiles(dirInit,stemInitFiles,stemDAMdir,monitorFormat)

# GATHER data on all RIL duplications
ril2arb <- ReadInFiles.func(dirRILdup)

# SET file to hold all output monitor tube data
# SET output object to hold summary data from all monitors
AllMonitorDat <- NULL
AllSummaryDat <- NULL

# LOOP through all monitors
for(i in 1:nrow(MonitorInfo)) {

    # INPUT activity monitor file
    # VALIDATE the file
    # REMOVE (to NAs) data from tubes that did not get filled
    # REMOVE (to NAs) data from tubes with low activity
    # ELIMINATE spurious activity spikes occurring at "lights on"
    monitorFname <- paste(dirOut,MonitorInfo[i,1],"/",MonitorInfo[i,2],sep="")
    MonDat <- InputMonitorFile(monitorFname,monitorFileNCols,stripCols)
    ValidDat <- CheckDupsIntervalsGaps(MonDat,readInterval,readIntUnits,MonitorInfo[i,],ErrorFile)
    ValidDat <- RemoveBadTubes(ValidDat,MonitorInfo[i,])
    ValidDat <- RemoveLowActivityTubeData(ValidDat,FindLowActivityTubes(ValidDat,activityThresh))
    ValidDat <- RemoveSpuriousLightsOnActivity(ValidDat)
    
    # CALCULATE time of death for each tube in monitor
    DeathDat <- DeathTimes(ValidDat,readInterval,readIntUnits,
        MaxNumSequentialInactiveBins,MaxNumActiveBins)

    # POPULATE a matrix with details (start time, genotype, experimenter) of
    #   fly in each tube directly from 'MonitorInfo'
    TubeDetailsDat <- PopulateIDMatrix(MonitorInfo[i,],monitorFormat)

    # GET actual RIL barcodes
    RIL <- as.character(unlist(lapply(TubeDetailsDat[,"Genotype"],function(X) FindRILBarcode(X,ril2arb))))
    
    # CALCULATE lifespan for each tube
    # DETERMINE whether fly is still alive at end of experiment
    LifespanDat <- Lifespan(TubeDetailsDat,DeathDat,"hours")
    StillAliveDat <- FliesAreAlive(ValidDat,EndTestWindow,EndActivityCount)
    
    # PULL TOGETHER all current monitor data
    curMonitorDat <- cbind(TubeDetailsDat,RIL,DeathDat,LifespanDat,StillAliveDat)

    # DETERMINE whether all flies from given genotype on current monitor
    #   were dead at end of experiment (if NOT this replicate of the genotype
    #   should be ignored)
    curMonitorDat <- cbind(curMonitorDat,AreAllFliesDead(curMonitorDat,aliveThresh))
    
    # APPEND current monitor tube data to 'AllMonitorDat'
    # GENERATE summary of data on monitor, and APPEND to 'AllSummaryDat'
    AllMonitorDat <- rbind(AllMonitorDat,curMonitorDat)
    AllSummaryDat <- rbind(AllSummaryDat,SummarizeMonitorData(curMonitorDat))
    
    # MAKE activity barplots
    # -> CHECK these to see if code is extracting correct time-of-death
    ActivityBarplot8x4(ValidDat,DeathDat,MonitorInfo[i,],dirActivityBarplot)
    #ActivityBarplot1x32(ValidDat,DeathDat,MonitorInfo[i,],dirActivityBarplot)
}

# OUTPUT all monitor tube data
write.table(AllMonitorDat,file=paste(dirSummary,"AllTubeData.txt",sep=""),
    col.names=NA,sep="\t",quote=F)


		# NOTE:
		# Checked all activity plots manually (9 March, 2021) and there are
		# no issues with spurious activity peaks


############################
###         GET          ###
###      SUMMARIES       ###
############################

# READ 'AllMonitorDat' back in from file
# NOTE: When read back in no longer have monitor tube name (e.g., "m01")
#       as rownames, instead have as new 1st column ("X")
AllMonitorDat <- read.table(paste(dirSummary,"AllTubeData.txt",sep=""),header=T,sep="\t")

# REMOVE any rows (i.e., tubes) from 'AllMonitorDat'
# where the software elected to
# remove individual from experiment ("NA")
# -> 30 NAs, 930 dead/alive
# -> keeping 14 alives since turned off monitors _just_ at end experiment
truncAllMonitorDat <- AllMonitorDat[!is.na(AllMonitorDat[,"AliveOrDead"]),]

# REMOVE any rows (i.e., tubes) from 'AllMonitorDat' where fly is
# still alive at end experiment OR where the software elected to
# remove individual from experiment ("NA")
#truncAllMonitorDat <- AllMonitorDat[AllMonitorDat[,"AliveOrDead"]!="alive"
#	& !is.na(AllMonitorDat[,"AliveOrDead"]),]
# REMOVE any rows still containing an NA value
# -> 2 of these
truncAllMonitorDat <- RemoveRowsWithNAs(truncAllMonitorDat)

# LIST unique 6-digit barcodes employed in study (these are
#   the replicate vials)
# GATHER number of rows associated with each (# of replicate
#   flies tested)
# LIST replicates with sufficient tested flies
# KEEP just these replicates
#   (no change)
uniqueGeno <- unique(truncAllMonitorDat[,"Genotype"])
uniqueGenoNum <- as.numeric(table(match(truncAllMonitorDat[,"Genotype"],uniqueGeno)))
KeepGeno <- uniqueGeno[uniqueGenoNum >= flyThresh]
truncAllMonitorDat <- truncAllMonitorDat[match(truncAllMonitorDat[,"Genotype"],
    KeepGeno,nomatch=0)!=0,]

# OUTPUT truncated monitor tube data
write.table(truncAllMonitorDat,file=paste(dirSummary,"TruncTubeData.txt",sep=""),
	col.names=NA,sep="\t",quote=F)

# EXTRACT list of unique cross genotypes
# -> 19 unique genotypes, 11 "1099", 8 "25374"
uniqueRIL <- sort(unique(truncAllMonitorDat[,"RIL"]))

# LOOP through unique RILs
# GET overall RIL summary stats, as well
#    as per "replicate" summary stats,
#    where a set of tubes in a single monitor are
#    considered a replicate (since came from single
#    replicate source cross vial)
FinalRILDat <- NULL
FinalRepDat <- NULL
RepFileLineCounter <- 1
for(i in 1:length(uniqueRIL)) {

    # EXTRACT all tube data for single RIL
    # SUMMARIZE and OUTPUT
    rilDAT <- truncAllMonitorDat[truncAllMonitorDat[,"RIL"]==uniqueRIL[i],]
    rilStats <- SummarizeValues(rilDAT[,"Lifespan.hours"])
    FinalRILDat <- rbind(FinalRILDat,rilStats)
    rownames(FinalRILDat)[i] <- as.character(uniqueRIL[i])
    
    # COUNT number of replicate tests for RIL
    # IF =1, do nothing. IF >1 get summaries
    #   per replicate
    rilReps <- unique(rilDAT[,"Monitor"])
    NumReps <- length(rilReps)
    if(NumReps==1) {
        # DO NOTHING
    }
    else {
        for(j in 1:NumReps) {
            # EXTRACT tube data for single replicate
            # SUMMARIZE and OUTPUT
            rilREP <- rilDAT[rilDAT[,"Monitor"]==rilReps[j],]
            repStats <- SummarizeValues(rilREP[,"Lifespan.hours"])
            repStats <- c(rilREP[1,"Directory"],rilREP[1,"Monitor"],
                rilReps[j],rilREP[1,"Experimenter"],repStats)
            FinalRepDat <- rbind(FinalRepDat,repStats)
            rownames(FinalRepDat)[RepFileLineCounter] <- as.character(uniqueRIL[i])
            RepFileLineCounter <- RepFileLineCounter+1
        }
    }
}

# OUTPUT summary files
colnames(FinalRILDat) <- c("Nflies","MeanPheno","MedianPheno","SDPheno")
write.table(FinalRILDat,file=paste(dirSummary,"RILsummary.txt",sep=""),
    col.names=NA,sep="\t",quote=F)
colnames(FinalRepDat) <- c("Directory","Monitor","Genotype","Experimenter",
    "Nflies","MeanPheno","MedianPheno","SDPheno")
write.table(FinalRepDat,file=paste(dirSummary,"ReplicateSummary.txt",sep=""),
    col.names=NA,sep="\t",quote=F)


		# NOTE:
		# No obvious monitor-to-monitor shifts in phenotype.








############################
############################
###                      ###
###     PASTE IN ALL     ###
###      FUNCTIONS       ###
###                      ###
############################
############################


###################################
# FUNCTION
###################################
# GENERIC, simple function return set
# of summary stats (number, mean, median,
# SD) for a vector of input values
###################################

SummarizeValues <- function(X) {
    X <- as.numeric(X)
    N <- length(X[!is.na(X)])
    meanX <- mean(X,na.rm=T)
    medX <- median(X,na.rm=T)
    sdX <- sd(X,na.rm=T)
    return(c(N,meanX,medX,sdX))
}


###################################
# FUNCTION
###################################
# GENERIC function to remove
# NA-containing rows from a matrix
###################################

RemoveRowsWithNAs <- function(FILE) {
    # COUNT number of NA values per row
    tmp <- as.numeric(apply(FILE,1,function(E) length(E[is.na(E)])))
    # KEEP only NA-free rows, output
    Output <- FILE[tmp==0,]
    return(Output)
}


###################################
# FUNCTION
###################################
# READS in data from single monitor (DAT)
# and a zero (or some other value)
# representing the number of alive flies
# per genotype that is acceptable
#
# RETURNS "1" if all flies from that genotype
# are dead, and a "0" otherwise
###################################

AreAllFliesDead <- function(DAT,COUNT) {
    # DETERMINE genotypes represented on monitor
    genoCol <- DAT[,"Genotype"]
    genos <- unique(genoCol)
    genos <- genos[!is.na(genos)]
    numGeno <- length(genos)
    
    # SET temporary live fly coding object
    tmpCode <- rep(NA,numGeno)

    # LOOP through genotypes
    for(i in 1:numGeno) {
        # PULL OUT subset of data associated with single genotype
        # COUNT number of "alive" values in 'AliveOrDead' column
        subDAT <- DAT[DAT[,"Genotype"]==genos[i] & !is.na(DAT[,"Genotype"]),]
        AliveOrDead <- subDAT[,"AliveOrDead"]
        numAlive <- length(AliveOrDead[AliveOrDead=="alive" & !is.na(AliveOrDead)])

        # IF more "alive" than threshold ('COUNT') return 0, otherwise return 1
        if(numAlive>COUNT) {
            tmpCode[i] <- 0
        }
        else {
            tmpCode[i] <- 1
        }
        # GENERATE vector showing that block of tubes from same
        # monitor holding same genotype all died ("1") or not ("0")
        genoCol[genoCol==genos[i] & !is.na(genoCol)] <- tmpCode[i]
    }
    
    # RETURN
    genoCol[genoCol!=0 & genoCol!=1] <- NA
    OUTPUT <- matrix(genoCol,ncol=1)
    colnames(OUTPUT) <- "AllDead"
    rownames(OUTPUT) <- names(genoCol)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READS in single arbitrary barcode,
# and 2-column RIL duplication matrix
#
# RETURNS associated RIL barcode
###################################

FindRILBarcode <- function(X,BARCODES) {
    # SIMPLE match function
    # -> assumes 2-column RIL duplication
    #    file with 1st col = RILbc and 2nd
    #    col = ARBbc of duplicated vial
    RILBC <- BARCODES[match(X,BARCODES[,2]),1]
    return(RILBC)
}


###################################
# FUNCTION
###################################
# READS in single element of 'AllMonitorDat'
# (data from one monitor)
#
# RETURNS summary information from each genotype
# on that monitor
###################################

SummarizeMonitorData <- function(DAT) {
    # EXTRACT directory/monitor info
    curDir <- DAT[1,"Directory"]
    curMon <- DAT[1,"Monitor"]
    
    # DETERMINE genotypes represented on monitor
    genos <- unique(DAT[,"Genotype"])
    genos <- genos[!is.na(genos)]
    numGeno <- length(genos)
    
    # MAKE output matrix based on this information
    OutputMat <- matrix(rep(NA,(numGeno*11)),ncol=11)
    colnames(OutputMat) <- c("Directory","Monitor","Genotype","RIL",
        "Experimenter","N","N.alive","N.dead","MeanLifespan",
        "MedianLifespan","SDLifespan")
        
    # LOOP through genotypes
    for(i in 1:numGeno) {
    
        # FILL in directory/monitor info
        OutputMat[i,1] <- curDir
        OutputMat[i,2] <- curMon
       
        # EXTRACT just tubes for given genotype
        # GET current genotype
        # GET associated RIL barcode
        # GET current experimenter
        subDAT <- DAT[DAT[,"Genotype"]==genos[i] & !is.na(DAT[,"Genotype"]),]
        curGeno <- subDAT[1,"Genotype"]
        OutputMat[i,3] <- curGeno
        OutputMat[i,4] <- subDAT[1,"RIL"]
        OutputMat[i,5] <- subDAT[1,"Experimenter"]
        
        # EXTRACT lifespan data
        # GET N, mean, median, SD
        lifeDAT <- as.numeric(subDAT[,"Lifespan.hours"])
        lifeDAT <- lifeDAT[!is.na(lifeDAT)]
        OutputMat[i,6] <- length(lifeDAT)
        OutputMat[i,9] <- mean(lifeDAT)
        OutputMat[i,10] <- median(lifeDAT)
        OutputMat[i,11] <- sd(lifeDAT)
        
        # EXTRACT alive/dead data
        # GET N alive and N dead
        adDAT <- subDAT[,"AliveOrDead"]
        adDAT <- adDAT[!is.na(adDAT)]
        OutputMat[i,7] <- length(adDAT[adDAT=="alive"])
        OutputMat[i,8] <- length(adDAT[adDAT=="dead"])
    }
    return(OutputMat)
}


###################################
# FUNCTION
###################################
# GENERIC function to read in files
# of same type (given the path to the
# directory) and output a single file
# concatenating all input files using
# 'rbind'
###################################

ReadInFiles.func <- function(PATH) {
    FileNames <- list.files(path=PATH)
    Output <- NULL
    for(i in FileNames) {
        tmpFile <- read.csv(paste(PATH,i,sep=""),header=T,sep="\t")
        Output <- rbind(Output,tmpFile)
    }
    return(Output)
}


###################################
# FUNCTION
###################################
# READ in monitor data, a test window
# at the end of the file (SPAN in bins)
# to check for activity, and an acceptable
# number of counts to consider the fly
# dead
#
# RETURN matrix of "alive" or "dead"
# or "NA"
#
# NOTE edited in December 2016 to change code
# now _assumes_ scanned tubes every 1 minute
###################################

FliesAreAlive <- function(DAT,SPAN,THRESH) {
    # TRANSFORM just date/time columns into something R likes
    AllDateTime <- apply(DAT[,1:2],1,function(bbb) paste(bbb[1],bbb[2],sep=" "))

    # EXTRACT the last date/time point at which the monitor
    #   had a "status" of 1 (i.e., not 24 or 51)
    # GENERATE date/time SPAN minutes before
    tmpADT <- AllDateTime[DAT[,"status"]==1]
    lastTime <- as.character(tmpADT[length(tmpADT)])

	# ADDED December 2016
	# SOME CHANGE TO R meant that (I think) now R recognizes the time
	# zone change. For one of the boric acid monitor runs, where
	# at some point time went back/forward 1 hour, the code gives NA.
	# But can fix by just using fact that each row in file is 1 minute
	# so can simply get 'firstTime' as some number of rows before 'lastTime'
#    firstTime <- strptime(lastTime,"%Y-%m-%d %H:%M:%S")-(SPAN*60)
    firstTime <- as.character(tmpADT[(length(tmpADT)-SPAN)])

    # GENERATE submatrix of 'DAT' between the first and last times
    endDAT <- DAT[AllDateTime>firstTime & AllDateTime<=lastTime,]
    # ACTIVITY counts in this period
    # VECTOR determining whether entire period is NA's (then return
    # NA value)
    # MAKE a vector stating whether fly is alive or not (based on
    #   input activity threshold)
    # RETURN as matrix
    endCounts <- apply(endDAT[,10:41],2,function(X) sum(X,na.rm=T))
    naCol <- apply(endDAT[,10:41],2,function(Y)
        if(length(Y[is.na(Y)])==length(Y)) {return(NA)} else {return(1)})
    aliveVector <- endCounts
    aliveVector[endCounts<=THRESH] <- "dead"
    aliveVector[endCounts>THRESH] <- "alive"
    aliveVector[is.na(naCol)] <- NA
    OUTPUT <- matrix(aliveVector,ncol=1)
    colnames(OUTPUT) <- "AliveOrDead"
    rownames(OUTPUT) <- names(aliveVector)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in tube details matrix and death
#   data matrix, and desired units of
#   returned time difference data
#
# RETURNS vector of times (1 per tube)
###################################

Lifespan <- function(TUBE,DEATH,UNITS) {
    # EXTRACT just the time data from the input matrices
    startTime <- TUBE[,"StartTimestamp"]
    deathTime <- DEATH[,"TimeOfDeath"]
    # GET difference between then in defined units
    # TURN into 1-column matrix and return
    TimeDiff <- apply(cbind(deathTime,startTime),1,function(AA)
        as.numeric(difftime(AA[1],AA[2],units=UNITS)))
    OUTPUT <- matrix(TimeDiff,ncol=1)
    colnames(OUTPUT) <- paste("Lifespan",UNITS,sep=".")
    rownames(OUTPUT) <- names(TimeDiff)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in current row of 'MonitorInfo'
#   file, and a FORMAT
# Possible 'FORMAT' values are "2halves"
#   (meaning using A/B barcodes) or
#   "4quadrants" (using 1/2/3/4 barcodes)
# RETURNS a matrix defining the monitor
#   details (directory/monitor), starting
#   time, genotype, and experimenter
#   for each tube
###################################

PopulateIDMatrix <- function(X,FORMAT) {
    # BOTH monitor sides filled with different genotypes
    # (potentially at different times by different people)
    if(FORMAT=="2halves") {
        # EXTRACT information
        TheDir <- X["DAMdir"]
        TheMon <- X["MonFname"]
        timeA <- X["Timestamp.A"]
        genoA <- X["Genotype.A"]
        exptA <- X["Experimenter.A"]
        timeB <- X["Timestamp.B"]
        genoB <- X["Genotype.B"]
        exptB <- X["Experimenter.B"]
        # DEFINE tube positions for A and B sides
        TubeData <- c("B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A")
        # DEVELOP categorization vectors
        rDir <- rep(TheDir,32)
        rMon <- rep(TheMon,32)
        rTime <- TubeData
        rTime[TubeData=="A"] <- timeA
        rTime[TubeData=="B"] <- timeB
        rGeno <- TubeData
        rGeno[TubeData=="A"] <- genoA
        rGeno[TubeData=="B"] <- genoB
        rExpt <- TubeData
        rExpt[TubeData=="A"] <- exptA
        rExpt[TubeData=="B"] <- exptB
        # GENERATE matrix
        OUT <- cbind(rDir,rMon,rTime,rGeno,rExpt)
        colnames(OUT) <- c("Directory","Monitor","StartTimestamp","Genotype","Experimenter")
        rownames(OUT) <- names(X)[3:34]
    }
    else {
        # EXTRACT information
        TheDir <- X["DAMdir"]
        TheMon <- X["MonFname"]
        time1 <- X["Timestamp.1"]
        geno1 <- X["Genotype.1"]
        expt1 <- X["Experimenter.1"]
        time2 <- X["Timestamp.2"]
        geno2 <- X["Genotype.2"]
        expt2 <- X["Experimenter.2"]
        time3 <- X["Timestamp.3"]
        geno3 <- X["Genotype.3"]
        expt3 <- X["Experimenter.3"]
        time4 <- X["Timestamp.4"]
        geno4 <- X["Genotype.4"]
        expt4 <- X["Experimenter.4"]
        # DEFINE tube positions for 1, 2, 3, and 4 quadrants
        TubeData <- c("1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4")
        # DEVELOP categorization vectors
        rDir <- rep(TheDir,32)
        rMon <- rep(TheMon,32)
        rTime <- TubeData
        rTime[TubeData=="1"] <- time1
        rTime[TubeData=="2"] <- time2
        rTime[TubeData=="3"] <- time3
        rTime[TubeData=="4"] <- time4
        rGeno <- TubeData
        rGeno[TubeData=="1"] <- geno1
        rGeno[TubeData=="2"] <- geno2
        rGeno[TubeData=="3"] <- geno3
        rGeno[TubeData=="4"] <- geno4
        rExpt <- TubeData
        rExpt[TubeData=="1"] <- expt1
        rExpt[TubeData=="2"] <- expt2
        rExpt[TubeData=="3"] <- expt3
        rExpt[TubeData=="4"] <- expt4
        # GENERATE matrix
        OUT <- cbind(rDir,rMon,rTime,rGeno,rExpt)
        colnames(OUT) <- c("Directory","Monitor","StartTimestamp","Genotype","Experimenter")
        rownames(OUT) <- names(X)[3:34]
    }
    return(OUT)
}


###################################
# FUNCTION
###################################
# READ in DAT (ValidDat - where unwanted
# monitor tubes should be NA'd out), DEATH
# (DeathDat), current monitor info row,
# and an output directory for plot
#
# MAKES single PDF per monitor (with
# individual plots per tube)
#
# Activity data simply a barplot, with the
# lifespan determined by my code as a red
# line, and the last activity bin as a blue
# triangle
###################################

ActivityBarplot8x4 <- function(DAT,DEATH,MINFO,OUTDIR) {
    # GENERATE plotfile output name
    # OPEN plot
    # LOOP through tubes in 'DAT'
    nameOut <- paste(OUTDIR,MINFO[1],".",gsub("txt","pdf",MINFO[2]),sep="")
    pdf(nameOut,width=25,height=13.8)
    pX <- matrix(c(1,9,17,25,2,10,18,26,3,11,19,27,4,12,20,28,5,13,21,29,6,
        14,22,30,7,15,23,31,8,16,24,32),ncol=4,byrow=T)
    layout(pX)
    for(i in 10:41) {
        plotDAT <- DAT[,i]
        # ONLY make plot if column not full of NA values
        if(length(plotDAT[is.na(plotDAT)])<length(plotDAT)) {
            # ENSURE the lifespan line always in relatively the same
            # position along x-axis
            maxDat <- max(plotDAT,na.rm=T)
            LifeDuration <- -maxDat/30
            # Activity barplot
            par(mar=c(2.5,5,1,1))
            X <- barplot(plotDAT,las=1,ylab="Activity Per Bin")
            mtext(colnames(DAT)[i],side=1,line=1)
            # Lifespan line
            lines(c(X[1],X[as.numeric(DEATH[(i-9),2])]),c(LifeDuration,LifeDuration),
                col="red",lwd=3,xpd=T)
            # Last active bin triangle
            points(X[max((1:length(plotDAT))[!is.na(plotDAT) & plotDAT>0])],(LifeDuration*2.5),
                pch=17,col="blue",cex=1,xpd=T)
        }
        else {
            par(mar=c(2.5,5,1,1))
            barplot(0,ylim=c(0,20),las=1,ylab="Activity Per Bin")
            mtext(colnames(DAT)[i],side=1,line=1)
        }
    }
    graphics.off()
}

ActivityBarplot1x32 <- function(DAT,DEATH,MINFO,OUTDIR) {
    # GENERATE plotfile output name
    # OPEN plot
    # LOOP through tubes in 'DAT'
    nameOut <- paste(OUTDIR,MINFO[1],".",gsub("txt","pdf",MINFO[2]),sep="")
    pdf(nameOut,width=8,height=4)
    for(i in 10:41) {
        plotDAT <- DAT[,i]
        # ONLY make plot if column not full of NA values
        if(length(plotDAT[is.na(plotDAT)])<length(plotDAT)) {
            # ENSURE the lifespan line always in relatively the same
            # position along x-axis
            maxDat <- max(plotDAT,na.rm=T)
            LifeDuration <- -maxDat/30
            # Activity barplot
            X <- barplot(plotDAT,las=1,ylab="Activity Per Bin",
                main=paste("./ ",MINFO[1]," / ",MINFO[2],sep=""),
                xlab=colnames(DAT)[i])
            # Lifespan line
            lines(c(X[1],X[as.numeric(DEATH[(i-9),2])]),c(LifeDuration,LifeDuration),
                col="red",lwd=4,xpd=T)
            # Last active bin triangle
            points(X[max((1:length(plotDAT))[!is.na(plotDAT) & plotDAT>0])],(LifeDuration*2),
                pch=17,col="blue",cex=1,xpd=T)
        }
        else {
            # DO NOTHING
        }
    }
    graphics.off()
}


###################################
# FUNCTION
###################################
# READ in DAT (ValidDat - where unwanted
# monitor tubes should be NA'd out), RI/RIU
# (read interval and units), and two arbitrary
# parameters to be passed to 'TubeDeathTime'
# function. Function essentially just
# wrapper for this other function.
#
# DOES generate the date/timepoint of the
# last status=1 row in the file (basically
# point at which monitor unplugged or
# stopped recording activity data)
#
# RETURNS matrix of time of death per tube
###################################

DeathTimes <- function(DAT,RI,RIU,MNSIB,MNAB) {
    # EXTRACT the date/time columns
    # EXTRACT just the monitor tube activity data
    DTVECT <- DAT[,1:2]
    ACTDAT <- DAT[,10:41]
    
    # APPLY FUNCTION through the tube data
    TimeOfDeath <- matrix(apply(ACTDAT,2,function(X)
        TubeDeathTime(X,DTVECT,RI,RIU,MNSIB,MNAB)),ncol=3,byrow=T)
    colnames(TimeOfDeath) <- c("TimeOfDeath","RowDeath","NumBinsRemoved")
    rownames(TimeOfDeath) <- c(paste("m0",seq(1,9,1),sep=""),paste("m",seq(10,32,1),sep=""))
    return(TimeOfDeath)
}


###################################
# FUNCTION
###################################
# PROVIDE validated input monitor file
# and row of 'MonitorInfo' file pertaining
# to current file
#
# RETURNS monitor file with any tubes having
# a (bad) 0 value in row turned to an NA vector
###################################

RemoveBadTubes <- function(DAT,MROW) {
    # EXTRACT just the tubes' 1(good)/0(bad) codes
    # EXTRACT names of any tubes that are bad
    tCodes <- MROW[3:34]
    badTubes <- names(tCodes[tCodes=="0"])
    # IF no bad tubes, do nothing, otherwise
    # NA out those columns in the monitor file
    if(length(badTubes)==0) {
        # DO NOTHING
    }
    else {
        DAT[,match(badTubes,colnames(DAT))] <- NA
    }
    return(DAT)
}


###################################
# FUNCTION
###################################
# PROVIDE path to DAM initiation file directory,
#   the text in the stem of each initiation file,
#   and the text in the stem of each DAM output
#   file directory
#
# ONLY possibilities for 'FORMAT' are "2halves"
#   and "4quadrants"
#
# RETURNS a file with one line per monitor (over
#   entire expt) including the directory where the
#   monitor file is, its filename, whether both
#   A and B sides were used, timestamps, genotypes,
#   and experimenters for each side
###################################

CollateInitiationFiles <- function(IDIR,INIT.STEM,DIR.STEM,FORMAT) {
    # LIST files in directory
    fileList <- list.files(IDIR)
    # SET an output file
    OUTPUT <- NULL

    # LOOP through and read in / process files
    for(i in 1:length(fileList)) {

        # READ in and sort by "Monitor.Barcode" column
        # GENERATE name of DAM output file directory associated
        #   with this initiation file
        curFile <- read.delim(paste(IDIR,fileList[i],sep=""),header=T)
        curFile <- curFile[order(curFile[,3]),]
        DamFileDirectory <- gsub(INIT.STEM,DIR.STEM,fileList[i])
        DamFileDirectory <- gsub(".txt","",DamFileDirectory)
        
        # GET sorted list of all monitors used
        mColumn <- as.character(curFile[,"Monitor.Barcode"])
        mSides <- unlist(lapply(mColumn,function(X)
            strsplit(X,split=".",fixed=T)[[1]][1]))
        mUsed <- sort(unique(mSides))

        # LOOP through them
        for(j in 1:length(mUsed)) {

            # GENERATE monitor filename (= "Monitor1.txt", etc)
            MonitorFilename <- paste("Monitor",as.numeric(strsplit(mUsed[j],
                split="M",fixed=T)[[1]][2]),".txt",sep="")

            # EXTRACT rows of 'curFile' containing data from given monitor
            # EXTRACT "tail" of barcode (will be A or B, or 1, 2, 3, or 4)
            monRows <- curFile[match(mSides,mUsed[j],nomatch=0)==1,]
            bcTail <- substr(monRows[,3],5,5)
            
            # GENERATE tube data (1=good, 0=bad/absent) based on
            # known loading pattern
            # -> Set vector of zeros
            # -> Fill in depending on 'bcTail' value
            TubeData <- rep(0,32)
            for(k in 1:length(bcTail)) {
                if(bcTail[k]=="A") {
                    TubeData[c(5:8,13:16,21:24,29:32)] <- rep(1,16)
                }
                if(bcTail[k]=="B") {
                    TubeData[c(1:4,9:12,17:20,25:28)] <- rep(1,16)
                }
                if(bcTail[k]=="1") {
                    TubeData[c(1,2,9,10,17,18,25,26)] <- rep(1,8)
                }
                if(bcTail[k]=="2") {
                    TubeData[c(3,4,11,12,19,20,27,28)] <- rep(1,8)
                }
                if(bcTail[k]=="3") {
                    TubeData[c(5,6,13,14,21,22,29,30)] <- rep(1,8)
                }
                if(bcTail[k]=="4") {
                    TubeData[c(7,8,15,16,23,24,31,32)] <- rep(1,8)
                }
            }

            # EXTRACT timestamps, genotypes, and experimenters
            Timestamp <- apply(monRows,1,function(X) TimestampConvertExcelToR(X[1]))
            Genotypes <- as.character(monRows[,2])
            Experimenters <- as.character(monRows[,4])
            
            # SOLVE PROBLEM
            # -> Possible that monitor not full, and need to fill
            #    correct timestamp, genotype, experimentor into
            #    correct cells of OUTPUT table
            if(FORMAT=="2halves") {
                if(length(bcTail)==2) {
                    # DO NOTHING
                }
                else {
                    if(bcTail=="A") {
                        Timestamp <- c(Timestamp,NA)
                        Genotypes <- c(Genotypes,NA)
                        Experimenters <- c(Experimenters,NA)
                    }
                    else {
                        Timestamp <- c(NA,Timestamp)
                        Genotypes <- c(NA,Genotypes)
                        Experimenters <- c(NA,Experimenters)
                    }
                }
            }
            else {
                if(length(bcTail)==4) {
                    # DO NOTHING
                }
                else {
                    Timestamp <- Timestamp[match(1:4,bcTail)]
                    Genotypes <- Genotypes[match(1:4,bcTail)]
                    Experimenters <- Experimenters[match(1:4,bcTail)]
                }
            }
            # COLLATE all data for current monitor (i.e., value of 'mUsed')
            OUTPUT <- rbind(OUTPUT,c(DamFileDirectory,MonitorFilename,TubeData,Timestamp,
                Genotypes,Experimenters))
        }
    }
    # ADD column names and return 'OUTPUT'
    if(FORMAT=="2halves") {
        colnames(OUTPUT) <- c("DAMdir","MonFname",paste("m0",seq(1,9,1),sep=""),
            paste("m",seq(10,32,1),sep=""),paste("Timestamp",c("A","B"),sep="."),
            paste("Genotype",c("A","B"),sep="."),paste("Experimenter",c("A","B"),sep="."))
    }
    else {
        colnames(OUTPUT) <- c("DAMdir","MonFname",paste("m0",seq(1,9,1),sep=""),
            paste("m",seq(10,32,1),sep=""),paste("Timestamp",1:4,sep="."),
            paste("Genotype",1:4,sep="."),paste("Experimenter",1:4,sep="."))
        
    }
    return(OUTPUT)
}
            

###################################
# FUNCTION
###################################
# READ Excel-based timestamp from
#   monitor initiation file
#
# RETURN an R-formated version
###################################

TimestampConvertExcelToR <- function(IN) {
    # CONVERT Excel-based timestamp in R flavor
    OUT <- strptime(as.character(IN),"%m/%d/%y %H:%M")
    OUT <- as.character(OUT)
    # PROBLEM: if timestamp exactly at midnight, R fails
    #   to put on time portion - do it here
    if(nchar(OUT)==19) {
        # DO NOTHING - this is correct
    }
    else {
        OUT <- paste(OUT,"00:00:00",sep=" ")
    }
    return(OUT)
}


###################################
# FUNCTION
###################################
# READ in validated monitor file (a 'ValidDat'
#   file), and list of columns that hold bad
#   data that you want to remove (or "None"
#   if no columns are bad)
#
# RETURN file with any columns of bad data
#   eliminated (converted to NA vector)
###################################

RemoveLowActivityTubeData <- function(MON,BADCOLS) {
    # IF NO 'bad' columns, simply return input 'MON' file
    if(BADCOLS[1]=="None") {
        # DO NOTHING
    }
    # OTHERWISE convert those columns to NA vectors
    else {
        MON[,match(BADCOLS,colnames(MON))] <- NA
    }
    # RETURN matrix
    return(MON)
}


###################################
# FUNCTION
###################################
# READ in validated monitor file (a 'ValidDat'
#   file), and a minimal acceptable activity
#   threshold (lower, and assume no fly or its
#   very sick)
# RETURN names of any such tubes
###################################

FindLowActivityTubes <- function(MON,ACTHRESH) {
    # CALCULATE total activity from each tube
    sumAct <- apply(MON[,10:41],2,function(X) sum(X,na.rm=T))
    # EXTRACT names of tubes with less than some
    # threshold number of activity counts
    lowActNames <- names(sumAct[sumAct<ACTHRESH])
    # RETURN these tube names (or "None")
    if(length(lowActNames)>0) {
        return(lowActNames)
    }
    else {
        return("None")
    }
}


###################################
# FUNCTION
###################################
# READ in binned activity counts from 1 tube (XACT),
#    the date/time columns from the binned datafile (DTVECT),
#    the bin interval and units (BI,BIU), and a threshold (TB)
#    giving the max number of inactive bins that can occur between
#    active bins before we decide the later activity is spurious, and
#    a number of active bins that can occur following the 'TB' sized
#    hiatus in activity ('BFE')
#
# -> The actual value of 'TB' is purely ad hoc
#
# OUTPUTs time/date of death for tube, and a 0/1 value showing
#    whether potentially spurious active bins were ignored
#
# NOTE: If a fly is alive when monitor is unplugged AND it is active
#       in the last bin in 'ValidDat' function will return an "NA"
#       for 'rowDeath' since the row-of-death is generated via a
#       match function as that row AFTER the fly shows last activity
#       count (and such a row won't exist in this case)
#       Did NOT solve problem since only result is to fail to give
#       red "lifespan" line on output plot. AND in any case these
#       individuals are alive, so the data from that genotype gets
#       ignored.
###################################

# NOTE:
# Might need to incorporate some additional thresholds: For instance,
#   one that only ignores late-occurring activity bins if the count is
#   very small, or if the late bins are isolated (they don't get
#   ignored if there are multiple late active bins

TubeDeathTime <- function(XACT,DTVECT,BI,BIU,TB,BFE) {
    # CHECK if 'XACT' holds (at least some) activity counts
    #   (if entire vector is NAs just return NA values)
    if(length(XACT[is.na(XACT)])<length(XACT)) {
        # GATHER only dates/times of bins for which activity
        #   count is >0
        # TRANSFORM into vector format R can work with
        #   [rm. any NA values that got passes in 'XACT' - could be
        #    result of selecting an experimental data window that
        #    includes bad monitor reads (that are NA'd out)]
        DTact <- DTVECT[XACT>0 & !is.na(XACT),]
        DTactv <- apply(DTact,1,function(AAA) paste(AAA[1],AAA[2],sep=" "))
    
        # GENERATE vector of # bins between each non-0 activity
        #   count bin (e.g., a 1 means fly is active in consecutive bins)
        #   -> vector is padded with a 0 at start
        BinSplit <- 0
        for(i in 1:(length(DTactv)-1)) {
            BinSplit <- c(BinSplit,difftime(DTactv[i+1],DTactv[i],units=BIU))
        }
        
        # CHECK whether any values in 'BinSplit' are greater than
        # input value of 'TB'
        if(sum((BinSplit>TB)*1,na.rm=T)==0) {
            # IF NO don't change date vector
            DTactv.t <- DTactv
        }
        else {
            # CHECK whether any value in final 'BFE' set of 'BinSplit'
            # values is >'TB'
            BinSplitEnd <- BinSplit[(length(BinSplit)-BFE+1):length(BinSplit)]
            EndOverTB <- match(1,(BinSplitEnd>TB)*1)
            if(is.na(EndOverTB)==TRUE) {
                # IF NO don't change date vector
                DTactv.t <- DTactv
            }
            else {
                # IF YES truncate date vector (so last value is time/date of
                # last activity BEFORE long inactive period)
                EndOverTB <- EndOverTB+length(BinSplit)-BFE
                DTactv.t <- DTactv[1:(EndOverTB)-1]
            }
        }
        
        # TEST whether any activity bins were removed
        if(length(DTactv)==length(DTactv.t)) {
            rmBin <- 0
        }
        else {
            rmBin <- 1
        }
        
        # EXTRACT the time of the bin that follows the one in which
        #   the fly was last active
        #   -> first define length of a bin
        #   -> also deal with potential to hit midnight, when R stops putting time
        #      onto output
        AddBin <- TimePeriodConversion(BI,BIU,"secs")
        TimeDeath <- as.character(strptime(as.character(DTactv.t[length(DTactv.t)]),
            "%Y-%m-%d %H:%M:%S")+AddBin)
        if(nchar(TimeDeath)==19) {
            # DO NOTHING
        }
        else {
            TimeDeath <- paste(TimeDeath,"00:00:00",sep=" ")
        }
        
        # EXTRACT the row of input 'DTVECT' corresponding to 'TimeDeath'
        reformatDTVECT <- apply(DTVECT,1,function(BBB) paste(BBB[1],BBB[2],sep=" "))
        rowDeath <- match(TimeDeath,reformatDTVECT)
        
        # OUTPUT time of death, row of death, and whether bins were removed
        return(c(TimeDeath,rowDeath,rmBin))
    }
    # IF input data all NAs
    else {
        return(c(NA,NA,NA))
    }
}


###################################
# FUNCTION
###################################
# READ in reformatted monitor datafile, 
#    the expected time between reads ('INTERVAL'),
#    the units of this time ('UNIT', one of
#    "secs", "mins", or "hours"), the current row
#    of the monitor info file, and the path to the
#    error file
# REMOVES read lines with exact duplicate date/time
#    stamps
# WRITES to error file if the monitor file includes
#   anything other then a set of sequential rows
#   each differing by INTERVAL UNIT time
# RETURNS monitor file
#
# NOTE that this function - the calculation of 'intReads'
# struggles when monitor run goes over time change. Does
# NOT affect lifespan data, but does give a ton of errors
# in output 'ErrorFile'
###################################

CheckDupsIntervalsGaps <- function(DAT,INTERVAL,UNIT,MROW,OUT) {
    # EXTRACT date and time from DAT into 2-col
    # 'as.character' matrix
    # REMOVE date and time columns from DAT
    DT <- cbind(DAT[,1],as.character(DAT[,2]))
    DAT <- DAT[,3:ncol(DAT)]

    # CHECK for exact duplicate date/time stamps
    # REMOVE any duplicates from two matrices
    Xdup <- duplicated(DT)*1
    DT2 <- DT[Xdup==0,]
    DAT2 <- DAT[Xdup==0,]

    # CALCULATE time difference between sequential reads
    nReads <- nrow(DAT2)
    DT2.both <- paste(DT2[,1],DT2[,2],sep=" ")
    intReads <- as.numeric(difftime(DT2.both[2:nReads],
        DT2.both[1:(nReads-1)],),units=UNIT)
    
    # CODE time differences (0 = interval expected, -1 = less,
    #    +1 = double, +2 = more, but not exactly double)
    # COUNT instances of the two types of gap
    intReadsCode <- c(as.numeric(lapply(intReads,function(A)
        if(A==INTERVAL) {return(0)} else {if(A==(2*INTERVAL))
        {return(1)} else {if(A>INTERVAL & A!=(2*INTERVAL))
        {return(2)} else {return(-1)}}})),0)
    countm1 <- length(intReadsCode[intReadsCode==-1])
    countp1 <- length(intReadsCode[intReadsCode==1])
    count2 <- length(intReadsCode[intReadsCode==2])
    
    # GENERATE error count line
    errorLine <- paste(MROW[1],MROW[2],countm1,countp1,count2,sep="\t")
    write(errorLine,OUT,append=T)
    
    # OUTPUT
    OUTPUT <- cbind(DT2,DAT2)
    colnames(OUTPUT)[1:2] <- c("date","time")
    rownames(OUTPUT) <- 1:nrow(OUTPUT)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in a value, and two time unit character
#    objects ("secs","mins","hours")
# CONVERT value from time in input units
#    to time in output units
###################################

TimePeriodConversion <- function(NUM,INUNIT,OUTUNIT) {
    # SET unit possibilities
    Upos <- c("secs","mins","hours")
    # CHECK both input units are correct
    # otherwise return NULL
    Ucheck <- sum(c(INUNIT==Upos,OUTUNIT==Upos))
    if(Ucheck!=2) {
        Output <- NULL
    }
    else {
        # GENERATE matrix of all possible combinations of
        # input and output units
        Ugrid <- expand.grid(Upos,Upos)
        # GENERATE vector of unit conversions that matches
        # format of matrix above
        Uconv <- c(1,60,3600,(1/60),1,60,(1/3600),(1/60),1)
        
        # FIND row of matrix that matches the input/output
        # unit values
        matchI <- match(Ugrid[,1],INUNIT,nomatch=0)
        matchO <- match(Ugrid[,2],OUTUNIT,nomatch=0)
        RowMatch <- which.max(matchI+matchO)
        
        # RETURN input value multipled by conversion factor
        Output <- NUM * Uconv[RowMatch]
    }
    return(Output)
}


###################################
# FUNCTION
###################################
# READ in monitor datafile
# CHECK it has correct num columns (via 'NC')
# ELIMINATE data for bad monitor reads
# STRIP unwanted/unused columns (via 'STRPCOLS')
# ADD column names
# REFORMAT date column to something more
#     familiar for R
###################################

InputMonitorFile <- function(FNAME,NC,STRPCOLS) {
    DAT <- read.delim(file=FNAME,header=F)
    # CHECK that number of columns of file is correct
    ncDAT <- ncol(DAT)
    if(ncDAT!=NC) {
        return(NULL)
    }
    else {
        # DEAL with 'bad' monitor reads
        # (convert data in all columns except
        #  index, date, time, and status to NA)
        DAT[DAT[,4]!=1,5:NC] <- rep(NA,(NC-4))
        
        # DEAL with possibility want to strip unused/unwanted
        # columns or not
        if(STRPCOLS=="yes") {
            # STRIP unwanted columns (keep date/time/monitor reads)
            DAT <- DAT[,-c(1,4:10)]
            # ADD useful column names
            colnames(DAT) <- c("date","time",
                paste(rep("m",9),1:9,sep="0"),
                paste(rep("m",23),10:32,sep=""))
        }
        if(STRPCOLS=="no") {
            # REMOVE index column only
            DAT <- DAT[,-1]
            # ADD useful column names
            colnames(DAT) <- c("date","time","status","extra.readings",
                "unused1","unused2","unused3","unused4","light.status",
                paste(rep("m",9),1:9,sep="0"),
                paste(rep("m",23),10:32,sep=""))
        }
        # REFORMAT date and output
        DAT[,1] <- as.character(strptime(DAT[,1],"%d %b %y"))
        return(DAT)
    }
}


###################################
# FUNCTION
###################################
# READ in validated monitor file (a 'ValidDat'
#   file), go to every row where the light has
#   just turned on, and remove spurious activity
#   spikes by taking 2 away from each activity value
#   in this row, and the preceding row
#
# RETURN file with spurious data eliminated
###################################

RemoveSpuriousLightsOnActivity <- function(MON) {
	# EXTRACT light status column (0=dark,1=light)
	# MAKE table with light status at minute n (col1) and light
	#    status at minute n+1 (col2)
	# GET row numbers when goes from dark (0 in col1) to light (1 in col2)
	#    and the preceding row
	# GET rows in 'MON'
	LightStat <- MON[,"light.status"]
	LS.mat <- cbind(LightStat[1:(length(LightStat)-1)],LightStat[2:length(LightStat)])
	LightTurnsOn <- apply(LS.mat,1,function(XX) if(!is.na(XX[1]) & XX[1]==0 & !is.na(XX[2]) & XX[2]==1) {return(1)} else {return(0)})
	RowsOfLightsOn <- sort(c((((1:nrow(LS.mat))[LightTurnsOn==1])),(((1:nrow(LS.mat))[LightTurnsOn==1])+1)))

	# LOOP through the 'ValidDat' data file for each of
	# the rows when the light first turns on
	#
	# REMOVE spurious '1' and '2' activity values for all monitor columns (m01-m32)
	for(i in 1:length(RowsOfLightsOn)) {
		# EXTRACT current activity values (vector of 32 length)
		CurVals <- as.numeric(MON[RowsOfLightsOn[i],10:41])
		# REMOVE '2' from all values (will create some -ve values)
		# ELIMINATE -1 and -2 values
		# INSERT new values back into 'ValidDat'
		ToRemove <- 2
		NewVals <- CurVals-ToRemove
		NewVals[NewVals==-1 | NewVals==-2] <- 0
		MON[RowsOfLightsOn[i],10:41] <- NewVals
	}
    # RETURN matrix
    return(MON)
}

##########################################################################################
##########################################################################################
# FUNCTION below not specifically for DAM analysis
# MAKES a standard looking, single-phenotype LOD curve plot


###########
# QTL Profile LOD plot
# Specifically for A*B crosses OR a single population
#
# SCANRES - 'DSPRscan' object
# XTHRESH and ATHRESH - numeric values
# PLOTNAME - output name of PDF (must include path)
###########

sjm.AxB.DSPRplot <- function(SCANRES,XTHRESH,ATHRESH,PLOTNAME) {

	# DEFINE maximum y-axis LOD score
	Ymax <- max(SCANRES$LODscores$LOD)
	
	# DEFINE axis text size
	CharExp <- 1.4

	# INITIATE pdf
	pdf(PLOTNAME,height=6,width=14)
	
	# CHROMOSOME X
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold
	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="X",]

	par(fig=c(0.03/5,1.42/5,0,1))
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.axis=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))
	mtext("LOD",side=2,cex=CharExp,line=3)

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,66.2),labels=c(NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,66.2),labels=c(0,66),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	axis(2,cex.axis=CharExp,las=1) # y=axis
	text((min(qLOD$Gpos)+2),(Ymax*0.95),
		expression(bold("X")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=XTHRESH,lty=2,col="black",lwd=1)

	# CHROMOSOME 2
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold

	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="2L" | SCANRES$LODscores[,"chr"]=="2R",]

	par(fig=c(1.02/5,3.21/5,0,1),new=TRUE)
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.lab=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,54,107.8),labels=c(NA,NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,54,107.8),labels=c(0,54,108),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	text((min(qLOD$Gpos)-0.5),(Ymax*0.95),
		expression(bold("2")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=ATHRESH,lty=2,col="black",lwd=1)

	# CHROMOSOME 3
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold

	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="3L" | SCANRES$LODscores[,"chr"]=="3R",]

	par(fig=c(2.81/5,5/5,0,1),new=TRUE)
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.lab=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,47.1,103),labels=c(NA,NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,47.1,103),labels=c(0,47,103),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	text((min(qLOD$Gpos)-0.5),(Ymax*0.95),
		expression(bold("3")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=ATHRESH,lty=2,col="black",lwd=1)
	
	graphics.off()
}
